import tensorflow as tf
import os
import random
import LoadTensorData

class RegModelLinear:
    """ A general implementation of a TensorFlow linear regression model. """

    verbose = False

    w = None
    b = None

    dataModelColCount = 0
    totalTrainingSteps = 10000
    trainStepsMultiplier = 1
    checkpoint = False
    randomSeed = False
    learning_rate = 0.0000001
    positiveResult = 5.0

    loadTensorData = None
    logPrint = 100
    evalType = ''

    def __init__(self, lVerbose=False, lLoadTensorData=None, lPositiveResult=0.50, lRandomSeed=False, lTrainStepsMultiplier=1.0, lLogPrint=100, lLearningRate=0.0000001, lEvalType=''):
        self.verbose = lVerbose
        self.learning_rate = lLearningRate
        self.loadTensorData = lLoadTensorData
        self.positiveResult = lPositiveResult
        self.randomSeed = lRandomSeed
        self.trainStepsMultiplier = lTrainStepsMultiplier
        self.dataModelColCount = self.loadTensorData.dataModelColCount
        self.logPrint = lLogPrint
        self.evalType = lEvalType
    # edef

    def inference(self, x):
        # Compute inference model over data x and return the result.
        return tf.matmul(x, self.w) + self.b
    # edef

    def loss(self, x, y):
        # Compute loss over training data x and expected outputs y.
        Y_predicted = self.inference(x)
        return tf.reduce_mean(tf.squared_difference(y, Y_predicted))
    # edef

    def inputs(self):
        # Read/generate input training data x and expected outputs y.
        return self.loadTensorData.train, self.loadTensorData.trainAnswers
    # edef

    def train(self, totalLoss):
        return tf.train.GradientDescentOptimizer(self.learning_rate).minimize(totalLoss)
    # edef

    def evaluate(self, sess, test_x, test_y):
        Y_predicted = self.inference(test_x)
        mse = tf.reduce_mean(tf.squared_difference(test_y, Y_predicted))
        print('Mean Squared Error: %.4f' % sess.run(mse))

        print('Custom Evaluation: %s' % (self.evalType))
        if self.evalType == 'weight_age':
            print(sess.run(self.inference([[80.0, 25.0]])))
            print(sess.run(self.inference([[65.0, 25.0]])))
        # eif
    # edef

    def startTraining(self):

        self.totalTrainingSteps = int(self.loadTensorData.trainCount * self.trainStepsMultiplier)
        print('Found training steps: %i' % self.totalTrainingSteps)

        with tf.Session() as sess:
            print('Found tensor dimension: %i' % self.dataModelColCount)
            if self.randomSeed == True:
                self.w = tf.Variable(tf.random_normal([self.dataModelColCount, 1], stddev=0.5), name='weights')
                self.b = tf.Variable(tf.random_normal([1], stddev=0.5), name='bias')
            else:
                self.w = tf.Variable(tf.zeros([self.dataModelColCount, 1]), name='weights')
                self.b = tf.Variable(0.00, name='bias')
            # eif

            # Model setup
            tf.global_variables_initializer().run()

            # Create a saver
            if self.checkpoint == True:
                saver = tf.train.Saver()
            # eif

            x, y = self.inputs()
            total_loss = self.loss(x, y)
            train_op = self.train(total_loss)

            coord = tf.train.Coordinator()
            threads = tf.train.start_queue_runners(sess, coord)
            training_steps = self.totalTrainingSteps

            initial_step = 0

            if self.checkpoint == True:
                # Verify we don't have a checkpoint saved already
                ckpt = tf.train.get_checkpoint_state(os.path.dirname(__file__) + "/checkpoints/")

                if ckpt and ckpt.model_checkpoint_path:
                    # Restores from checkpoint
                    saver.restore(sess, ckpt.model_checkpoint_path)
                    initial_step = int(ckpt.model_checkpoint_path.rsplit('-', 1)[1])
                # eif
            # eif

            # Training loop
            if self.checkpoint == True:
                for step in range(initial_step, training_steps):
                    sess.run([train_op])

                    if step % self.logPrint == 0:
                        print ("Loss: ", sess.run([total_loss]))
                    # eif

                    if step % 1000 == 0:
                        saver.save(sess, './checkpoints/eod-model', global_step=step)
                    # eif

                # efl
            else:
                for step in range(training_steps):
                    sess.run([train_op])

                    if step % self.logPrint == 0:
                        print ("Loss: ", sess.run([total_loss]))
                    # eif

                # efl
            # eif

            self.evaluate(sess, self.loadTensorData.validate, self.loadTensorData.validateAnswers)

        # ewith
    # edef

# eclass
